import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.*; 

/**
 * A modified JComponent that places an ellipse on the screen, allows
 * its color to be changed, and allows it to be moved around.
 * 
 * Simon Parsons
 * 4th November 2012
 *
 */
public class ColoredComponent extends JComponent 
	implements ActionListener
{   
	private int elX = 100, elY = 100;         
	private boolean xRight = true;
	private boolean yDown = true;
	
	private JButton theButton;                                           
	private JButton theOtherButton;     
	
	private Color theColor = Color.green;
	
	private ThreadOne t1;
	private ThreadTwo t2;
	
	// The constructor sets up two buttons
	public ColoredComponent() {
		theButton = new JButton("Start!");                      
		setLayout(new FlowLayout());                            
		add(theButton);
		theButton.addActionListener(this); 
		theOtherButton = new JButton("Stop!");
		add(theOtherButton);
		theOtherButton.addActionListener(new GoodBye());   
	}	
		
	// Here we allow access to two threads --- this allows
	// main to connect this object with the threads
	public void setThreads(ThreadOne t1, ThreadTwo t2){
		this.t1 = t1;
		this.t2 = t2;
	}
	
	// Paint just draws the ellipse at the location we want
	public void paintComponent(Graphics g){                        
		Graphics2D g2 = (Graphics2D)g;
		Shape c = new Ellipse2D.Float(elX, elY, 250, 150);         
		g2.setPaint(theColor);
		g2.fill(c);
	}

	// Allow the threads to modify the color of the ellipse.
	// Naturally this method has to be synchronized.
	public synchronized void changeColor(Color color){
		if (this.theColor != color){
			this.theColor = color;
		}
		repaint();
	}
	
	// Allow the threads to modify the position of the ellipse.
	// Again these methods are synchronized.
	public synchronized void changePositionX(){
		if(xRight && (elX > 140)){
			xRight = false;
		}
		
		if(!xRight && (elX < 10)){
			xRight = true;
		}
		
		if(xRight){
			elX+= 5;
		}
		else{
			elX-= 5;
		}	
		repaint();
	}
	
	public synchronized void changePositionY(){
		if(yDown && (elY >= 235)){
			yDown = false;
		}
		
		if(!yDown && (elY < 50)){
			yDown = true;
		}
		
		if(yDown){
			elY+= 5;
		}
		else{
			elY-= 5;
		}	
		repaint();
	}
	
	// The only listener we have to supply is for the start 
	// button. This kicks off the two threads.
	public void actionPerformed(ActionEvent e){
		// If the button is pressed, move the message
		if(e.getSource() == theButton){
			t1.start();
			t2.start();
		}
	}
	
}
